<?php
if (!class_exists('VoyaDespachosTracking')) {
  class VoyaDespachosTracking {
    const AJAX_ACTION = 'voya_seguimiento_pedido'; // ajax action
    const DOM_TARGET =  'voya_tracking_response'; // dom element to put the quotes
    const VOYA_SEGUIMIENTO_URL = 'https://app.voyapp.cl/api/tracking';
    
    function __construct() {
      add_shortcode('voya_seguimiento', array($this, 'addShortcode'));
      add_action( 'wp_ajax_'.self::AJAX_ACTION, array($this, 'getTracking'));
      add_action( 'wp_ajax_nopriv_'.self::AJAX_ACTION, array($this, 'getTracking'));
    }

    /**
     * Shortcode callback
     * @param $atts
     * @return string
     */
    public function addShortcode($atts){
      
        
        $out = '
                  <style>

                    ul.voya-list ul, li.voya-list{
                      list-style: none;
                      padding: 0;
                    }

                    .voya-timeline-container{
                      display: flex;
                      justify-content: center;
                      align-items: center;
                      padding: 0 1rem;
                      padding: 3rem 0;
                    }
                    .voya-wrapper{
                      background: #eaf6ff;
                      padding: 2rem;
                      border-radius: 15px;
                    }
                    h1.voya-heading{
                      font-size: 1.1rem;
                      font-family: sans-serif;
                      line-height: 1.5
                    }
                    .voya-sessions{
                      margin-top: 2rem;
                      border-radius: 12px;
                      position: relative;
                    }
                    li.voya-list{
                      padding-bottom: 1.5rem;
                      border-left: 1px solid #4274FF;
                      position: relative;
                      padding-left: 20px;
                      margin-left: 10px;
                    }
                    li.voya-list:last-child{
                      border: 1px solid transparent;
                      padding-bottom: 0;
                    }
                    li.voya-list:before{
                      content: "";
                      width: 15px;
                      height: 15px;
                      background: #4274FF;
                      border: 1px solid #4274FF;
                      border-radius: 50%;
                      position: absolute;
                      left: -8px;
                      top: 0;
                    }
                    .voya-time{
                      color: #2a2839;
                      font-family: "Poppins", sans-serif;
                      font-weight: 500;
                    }
                    li.voya-list p{
                      color: #4f4f4f;
                      font-family: sans-serif;
                      line-height: 1.5;
                      margin-top:0.4rem;
                    }
                    /*--------------FORM*/
                      .voya-form-inline input[type=text], .voya-form-inline select {
                        width: 100%;
                        padding: 12px 20px;
                        margin: 8px 0;
                        display: block;
                        border: 1px solid #ccc;
                        border-radius: 4px;
                        box-sizing: border-box;
                      }

                      .voya-form-inline button[type=button] {
                        width: 100%;
                        background-color: #4274FF;
                        color: white;
                        padding: 14px 20px;
                        margin: 8px 0;
                        border: none;
                        border-radius: 4px;
                        cursor: pointer;
                        font-weight: bold;
                        -webkit-transition: .5s all;   
                        -webkit-transition-delay: 5s; 
                        -moz-transition: .5s all;   
                        -moz-transition-delay: 5s; 
                        -ms-transition: .5s all;   
                        -ms-transition-delay: 5s; 
                        -o-transition: .5s all;   
                        -o-transition-delay: 5s; 
                        transition: .5s all;   
                        transition-delay: 5s; 
                      }

                      .voya-form-inline button[type=button]:hover {
                        background-image: linear-gradient(140deg, #4274FF 0%, #0EC0EA 100%);
                        -webkit-transition-delay: .5s;
                        -moz-transition-delay: .5s;
                        -ms-transition-delay: .5s;
                        -o-transition-delay: .5s;
                        transition-delay: .5s;
                      }
                      .voya-form-inline button[type=button]:focus {
                        background-image: linear-gradient(140deg, #4274FF 0%, #0EC0EA 100%);
                      }

                      div.voya-form-container {
                        border-radius: 5px;
                        background-color: #f2f2f2;
                        padding-top: 30px;
                        padding-left: 20px;
                        padding-right: 20px;
                        padding-bottom: 55px;
                      }
                      .voya-form-container .voya-powered-by{
                        font-size: 14px;
                        float: right;
                      }
                      .voya-form-container .voya-powered-by a{
                        color: #4274ff;
                        text-decoration: underline;
                      }
                  </style>
                  <div id="voya_tracking_error">
                  </div>
                  <div class="voya-form-container">
                    <form class="voya-form-inline" id="voya-form-tracking" action="#">
                      <label for="voya_tracking_code">Código de seguimiento:</label>
                      <input type="text" id="voya_tracking_code" name="tracking_code">
                      <label for="voya_courier">Courier:</label>
                      <select id="voya_courier" name="courier">
                        <option value="1">Starken</option>
                        <option value="5">Blue Express</option>
                      </select>
                      <button type="button" id="trackShipment">Consultar</button>
                    </form>
                    <p class="voya-powered-by">
                      Powered by <a href="https://voyapp.cl" target="_blank">Voya.</a>
                    </p>
                  </div>
                  <div id="voya_tracking_response">
                  </div>
                ';

        // loading js
        // jquery depends
        wp_enqueue_script('voya-tracking-js', VOYA_PLUGIN_URL.'/public/js/voya-tracking.js', array('jquery'),'1.0',true);
        // passing to js needed vars
        wp_localize_script( 'voya-tracking-js', 'ajaxParams',
            array(
                'ajaxurl'   => admin_url( 'admin-ajax.php'), // for frontend ( not admin )
                'action'    => self::AJAX_ACTION, //
            )
        );
        // render shortcode replacement
        return $out;
    }

    /**
     * Ajax Callback
     */
    public function getTracking(){
        echo $this->voyaQueryTracking();
        die();
    }

    /**
     * Getting random Qoute from the file
     * @param $path
     * @return mixed
     */
    public function voyaQueryTracking(){
        $headers = $this->getRequestHeaders();
        if(!$headers){
          return "No se ha establecido una API KEY para poder acceder a esta función.";
        }
        $data = $this->setRemotePostData();
        $result = wp_remote_post(self::VOYA_SEGUIMIENTO_URL, array(
            'headers' => $headers,
            'method' => 'POST',
            'body' => http_build_query($data),
            'timeout' => 13
          )
        );
        $validatedResponse = $this->responseValidation($result);
        if($validatedResponse["status"] == "error"){
          return $validatedResponse["message"];
        }
        return json_encode($validatedResponse);
    }

    private function setRemotePostData(){
      $data = [];
      $data["platform"] = "wp_wc";
      $valid = true;
      if(isset($_POST["tracking_code"]) && !empty($_POST["tracking_code"])){
        $data["tracking_code"] = $_POST["tracking_code"];
      }else{
        $valid = false;
      }
      if(isset($_POST["courier"]) && !empty($_POST["courier"])){
        $data["courier"] = $_POST["courier"];
      }else{
        $valid = false;
      }
      return $valid ? $data : false;
    }

    private function getRequestHeaders(){
      $options = get_option('woocommerce_voya_despachos_settings',[]);
      $apikey = '';
      if(isset($options['api'])){
        $apikey = $options['api'];
      }else{
        return false;
      }
      $headers = array();
      $headers["Authorization"] = "Bearer ".$apikey;
      $headers["Accept"] = "application/json";
      return $headers;
    }

    private function responseValidation($result){
      if (is_wp_error($result)) {
          $this->write_log("ERROR IS_WP_ERROR");
          $this->write_log($result);
          return ["status" => "error","message" => "Error de Wordpress/WooCommerce, por favor ponerse en contacto con el administrador del sitio web."];
        }

        $decoded = json_decode($result['body']);
        if (isset($decoded->status) && $decoded->status == 'ERROR') {
          $this->write_log("ERROR VOYA - MENSAJE INFORMATIVO:");
          $this->write_log($decoded->message);
          return ["status" => "error","message" => $decoded->message];
        }
        if(isset($decoded->message) && $decoded->message != "OK"){
          $msg = "";
          switch ($decoded->message) {
            case 'Unauthenticated.':
              $this->write_log("ERROR: PROBLEMA DE AUTENTICACIÓN EN EL SISTEMA VOYA.");
              $msg = "ERROR: PROBLEMA DE AUTENTICACIÓN EN EL SISTEMA VOYA.";
              break;
            case 'Too Many Attempts.':
              $this->write_log("ERROR: DEMASIADAS CONSULTAS CONCURRENTES.");
              $msg = "ERROR: DEMASIADAS CONSULTAS CONCURRENTES.";
              break;
            
            default:
               $this->write_log("ERROR: ".$decoded->message);
               $msg = "ERROR: ".$decoded->message;
              break;
          }
          return ["status" => "error","message" => $msg];
        }
        return ["status" => "success","response"=>$decoded];
    }

    private function write_log ( $log )  {
      if ( is_array( $log ) || is_object( $log ) ) {
        error_log( print_r( $log, true ) );
      } else {
        error_log( $log );
      }
    }
  }
}