<?php
/*
 * Plugin Name: Voya Chile - Lugares y Cotizador de Despachos
 * Plugin URI: https://voyapp.cl
 * Description: Añade las regiones y comunas de Chile a WooCommerce, además, podrás contar con un cotizador de despachos de múltiples couriers, de esta manera, tus clientes podrán pagar de inmediato el valor del envío. Este plugin es compatible con "MkRapel Regiones y Ciudades de Chile para WC".
 * Version: 1.5.1
 * Author: Onion Media
 * Author URI: https://onionmedia.cl
 * Requires PHP: 7.0 or greater
 * Requires at least: 5.6
 * Tested up to: 6.0
 * WC requires at least: 4.5
 * WC tested up to: 6.7.0
 */

if(!defined( 'ABSPATH' )) exit;

/**
 * Check if WooCommerce is active
 */
if(in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
  define( 'VOYA_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
  define( 'VOYA_PLUGIN_URL', plugin_dir_url(__FILE__) );
  /**
   * Check if MktRapel plugin active. If not, add locations
   */
  if(!in_array('wc-ciudades-y-regiones-de-chile/wc-ciudades-y-regiones-de-chile.php', apply_filters('active_plugins', get_option('active_plugins')))){
    function voyacl_despachos_destinos_init(){
      require_once ('includes/classes/voya_despachos_destinos.php');
      global $pagenow;
      $GLOBALS['wc_states_places'] = new WC_States_Places(__FILE__);
    }
    add_action('plugins_loaded','voyacl_despachos_destinos_init',1);
  }

  function voyacl_despachos_calculo_init(){
      require_once ('includes/classes/voya_despachos_calculo.php');
      $abdc = new Voya_Despachos_Calculo();
  }

  function voyacl_despachos_tracking(){
      require_once ('includes/classes/voya_despachos_tracking.php');
      $vsc = new VoyaDespachosTracking();
  }

  function voyacl_despachos_calculo($methods){
      $methods[] = 'Voya_Despachos_Calculo';
      return $methods;
  }
  
  function voyacl_settings_link( $links ) {
    $links[] = '<a href="' .
      admin_url( 'admin.php?page=wc-settings&tab=shipping&section=voya_despachos' ) .
      '">' . __('Settings') . '</a>';
    return $links;
  }

  function voyacl_custom_shipping_method_label( $label, $method ){
    if(strpos($method->method_id, 'voya_despachos') !== false) {
      $newLabel = "";
      $explodedLabel = explode(":",$label);
      if(array_key_exists("courier_titulo_alternativo",$method->meta_data) && $method->meta_data["courier_titulo_alternativo"] != ""){
        $explodedLabel[0] = $method->meta_data["courier_titulo_alternativo"];
        $newLabel = $explodedLabel[0].':'.$explodedLabel[1];
      }
      if(array_key_exists("dias_despacho",$method->meta_data) && $method->meta_data["dias_despacho"] != "N/D"){
        $newLabel = $explodedLabel[0].' - '.$method->meta_data["dias_despacho"].':'.$explodedLabel[1];
      }
      return $newLabel ? $newLabel : $label;
    }
    return $label;
  }

  function voyacl_order_shipping_to_display_shipped_via( $shipped_via, $order ) {
    $shippingData = $order->get_shipping_methods();
    $altTitle = null;
    foreach ($shippingData as $objectId => $singleObject) {
      if($singleObject->get_meta('courier_titulo_alternativo')){
        $altTitle = $singleObject->get_meta('courier_titulo_alternativo');
        break;
      }
    }
    if($altTitle){
      $shipped_via = '&nbsp;<small class="shipped_via">' . sprintf( __( 'vía %s', 'woocommerce' ), $altTitle) . '</small>';
    }
    return $shipped_via;
  }

  function voyacl_checkout_table_credits() {
    $options = get_option('woocommerce_voya_despachos_settings',[]);
    if(isset($options['display_powered_by_checkout']) && $options['display_powered_by_checkout'] == 'yes'){
      echo '<tr><td colspan="2" style="text-align: right;font-size: 9px;border:0;">Cotización de despachos por <a href="https://voyapp.cl" target="_blank">Voya.</a></td></tr><style>.woocommerce table.shop_table {border:0;}</style>';
    }
  }

  add_action('plugins_loaded','voyacl_despachos_tracking',1);
  add_action('woocommerce_shipping_init', 'voyacl_despachos_calculo_init');
  add_action('woocommerce_review_order_after_order_total', 'voyacl_checkout_table_credits' );
  add_filter('woocommerce_cart_shipping_method_full_label', 'voyacl_custom_shipping_method_label', 10, 2);
  add_filter('woocommerce_shipping_methods', 'voyacl_despachos_calculo');
  add_filter( 'woocommerce_order_shipping_to_display_shipped_via', 'voyacl_order_shipping_to_display_shipped_via', 10, 2 );
  add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'voyacl_settings_link');
};
